@@ -12,8 +12,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 import collections
-import mock
 import os
+from unittest import mock
 
 import ddt
 from oslo_concurrency import processutils as putils
@@ -932,7 +932,7 @@ Setting up iSCSI targets: unused
         mock_isdir.return_value = False
         expected = []
         actual = self.connector.get_all_available_volumes()
-        self.assertItemsEqual(expected, actual)
+        self.assertCountEqual(expected, actual)
 
     @mock.patch.object(iscsi.ISCSIConnector, '_get_device_path')
     def test_get_potential_paths_mpath(self, get_path_mock):
@@ -1113,43 +1113,11 @@ Setting up iSCSI targets: unused
         self.assertListEqual(expected_cmds, actual_cmds)
         get_sessions_mock.assert_called_once_with()
 
-    @mock.patch.object(iscsi.ISCSIConnector, '_iscsiadm_update')
-    @mock.patch.object(iscsi.ISCSIConnector, '_get_transport',
-                       return_value='default')
-    @mock.patch.object(iscsi.ISCSIConnector, '_get_iscsi_sessions_full')
-    @mock.patch('time.sleep')
-    def test_connect_to_iscsi_portal_fail_op_new(self, sleep_mock,
-                                                 get_sessions_mock,
-                                                 get_transport_mock,
-                                                 iscsiadm_update_mock):
-        get_sessions_mock.return_value = []
-        with mock.patch.object(self.connector, '_execute') as exec_mock:
-            exec_mock.side_effect = [('', 21), ('', 6), ('', 21), ('', 6),
-                                     ('', 21), ('', 6)]
-            self.assertRaises(exception.BrickException,
-                              self.connector._connect_to_iscsi_portal,
-                              self.CON_PROPS)
-        expected_cmds = ['iscsiadm -m node -T tgt1 -p ip1:port1',
-                         'iscsiadm -m node -T tgt1 -p ip1:port1 '
-                         '--interface default --op new',
-                         'iscsiadm -m node -T tgt1 -p ip1:port1',
-                         'iscsiadm -m node -T tgt1 -p ip1:port1 '
-                         '--interface default --op new',
-                         'iscsiadm -m node -T tgt1 -p ip1:port1',
-                         'iscsiadm -m node -T tgt1 -p ip1:port1 '
-                         '--interface default --op new']
-        actual_cmds = [' '.join(args[0]) for args in exec_mock.call_args_list]
-        self.assertListEqual(expected_cmds, actual_cmds)
-        iscsiadm_update_mock.assert_not_called()
-
-        # Called twice by the retry mechanism
-        self.assertEqual(2, sleep_mock.call_count)
-
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwn',
                        side_effect=(None, 'tgt2'))
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
     @mock.patch.object(iscsi.ISCSIConnector, '_cleanup_connection')
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     def test_connect_single_volume(self, sleep_mock, cleanup_mock,
                                    connect_mock, get_wwn_mock):
         def my_connect(rescans, props, data):
@@ -1174,7 +1142,7 @@ Setting up iSCSI targets: unused
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwn', return_value='')
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
     @mock.patch.object(iscsi.ISCSIConnector, '_cleanup_connection')
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     def test_connect_single_volume_no_wwn(self, sleep_mock, cleanup_mock,
                                           connect_mock, get_wwn_mock):
         def my_connect(rescans, props, data):
@@ -1202,7 +1170,7 @@ Setting up iSCSI targets: unused
                        side_effect=(None, 'tgt2'))
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
     @mock.patch.object(iscsi.ISCSIConnector, '_cleanup_connection')
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     def test_connect_single_volume_not_found(self, sleep_mock, cleanup_mock,
                                              connect_mock, get_wwn_mock):
 
@@ -1241,7 +1209,7 @@ Setting up iSCSI targets: unused
     @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_path')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_wwid')
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     def test_connect_multipath_volume_all_succeed(self, sleep_mock,
                                                   connect_mock, add_wwid_mock,
                                                   add_path_mock, get_wwn_mock,
@@ -1279,7 +1247,7 @@ Setting up iSCSI targets: unused
     @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_path')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_wwid')
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     def test_connect_multipath_volume_no_wwid(self, sleep_mock, connect_mock,
                                               add_wwid_mock, add_path_mock,
                                               get_wwn_mock, find_dm_mock):
@@ -1321,7 +1289,7 @@ Setting up iSCSI targets: unused
     @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_path')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_wwid')
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     def test_connect_multipath_volume_all_fail(self, sleep_mock, connect_mock,
                                                add_wwid_mock, add_path_mock,
                                                get_wwn_mock, find_dm_mock):
@@ -1348,7 +1316,7 @@ Setting up iSCSI targets: unused
     @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_path')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_wwid')
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     def test_connect_multipath_volume_some_fail_mp_found(self, sleep_mock,
                                                          connect_mock,
                                                          add_wwid_mock,
@@ -1390,7 +1358,7 @@ Setting up iSCSI targets: unused
     @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_wwid')
     @mock.patch.object(iscsi.time, 'time', side_effect=(0, 0, 11, 0))
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     def test_connect_multipath_volume_some_fail_mp_not_found(self, sleep_mock,
                                                              connect_mock,
                                                              time_mock,
@@ -1434,7 +1402,7 @@ Setting up iSCSI targets: unused
     @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_wwid')
     @mock.patch.object(iscsi.time, 'time', side_effect=(0, 0, 11, 0))
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
-    @mock.patch('time.sleep', mock.Mock())
+    @mock.patch('os_brick.utils._time_sleep', mock.Mock())
     def test_connect_multipath_volume_all_loging_not_found(self,
                                                            connect_mock,
                                                            time_mock,
@@ -1458,7 +1426,7 @@ Setting up iSCSI targets: unused
         find_dm_mock.assert_not_called()
         self.assertEqual(12, connect_mock.call_count)
 
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'scan_iscsi')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'device_name_by_hctl',
                        return_value='sda')
@@ -1491,7 +1459,7 @@ Setting up iSCSI targets: unused
         dev_name_mock.assert_called_once_with(mock.sentinel.session, hctl)
         sleep_mock.assert_called_once_with(1)
 
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'scan_iscsi')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'device_name_by_hctl',
                        side_effect=(None, None, None, None, 'sda'))
@@ -1522,7 +1490,7 @@ Setting up iSCSI targets: unused
         self.assertEqual(5, dev_name_mock.call_count)
         self.assertEqual(4, sleep_mock.call_count)
 
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'scan_iscsi')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'device_name_by_hctl',
                        side_effect=(None, None, None, None, 'sda'))
@@ -1577,7 +1545,7 @@ Setting up iSCSI targets: unused
         expected.update(failed_logins=1, stopped_threads=1)
         self.assertDictEqual(expected, data)
 
-    @mock.patch('time.sleep', mock.Mock())
+    @mock.patch('os_brick.utils._time_sleep', mock.Mock())
     @mock.patch.object(linuxscsi.LinuxSCSI, 'scan_iscsi')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'device_name_by_hctl',
                        return_value=None)
@@ -1608,7 +1576,7 @@ Setting up iSCSI targets: unused
             [mock.call(mock.sentinel.session, hctl),
              mock.call(mock.sentinel.session, hctl)])
 
-    @mock.patch('time.sleep', mock.Mock())
+    @mock.patch('os_brick.utils._time_sleep', mock.Mock())
     @mock.patch.object(linuxscsi.LinuxSCSI, 'scan_iscsi')
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_to_iscsi_portal')
     def test_connect_vol_stop_connecting(self, connect_mock, scan_mock):
@@ -1698,7 +1666,7 @@ Setting up iSCSI targets: unused
             mock.call('/dev/disk/by-id/dm-...'),
             mock.call('/dev/disk/by-id/scsi-wwn')])
 
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     @mock.patch('os.path.realpath', return_value='/dev/sdz')
     @mock.patch('os.listdir', return_value=['dm-...', 'scsi-...'])
     def test__get_device_link_not_found(self, listdir_mock, realpath_mock,
@@ -1714,7 +1682,7 @@ Setting up iSCSI targets: unused
                  mock.call('/dev/disk/by-id/scsi-...')])
         self.assertEqual(9, realpath_mock.call_count)
 
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     @mock.patch('os.path.realpath')
     @mock.patch('os.listdir', return_value=['dm-...', 'scsi-...'])
     def test__get_device_link_symlink_found_after_retry(self, mock_listdir,
@@ -1737,7 +1705,7 @@ Setting up iSCSI targets: unused
             + [mock.call('/dev/disk/by-id/scsi-wwn')])
         self.assertEqual(7, mock_realpath.call_count)
 
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     @mock.patch('os.path.realpath')
     @mock.patch('os.listdir', return_value=['dm-...', 'scsi-...'])
     def test__get_device_link_symlink_found_after_retry_by_listdir(
